/**
 * This file is part of my seminar project.
 * Following code, along with other files in this project,
 * realizes the visualization of A* algorithm.
 * 
 * @author Teon Banek
 */

package hr.fer.seminar;

import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JLabel;

/**
 * This class is a tile used in creating {@link TileMap}.
 * 
 * @author Teon Banek
 * 
 */
public class Tile {

	public static final int WIDTH = 32;
	public static final int HEIGHT = 32;

	private BufferedImage img = null;
	private int x;
	private int y;

	/**
	 * Flag: is this tile blocking movement.
	 */
	private boolean blocking = false;

	/**
	 * Flag: is this tile the start of a path.
	 */
	private boolean start = false;

	/**
	 * Flag: is this tile the goal of a path.
	 */
	private boolean goal = false;

	/**
	 * Flag: is this tile the next tile to be put in closed set by
	 * {@link Pathfinder}.
	 */
	private boolean next = false;

	/**
	 * Flag: is this tile in closed set.
	 */
	private boolean closed = false;

	/**
	 * Flag: is this tile part of the shortest path.
	 */
	private boolean path = false;

	/**
	 * Label used for displaying the F score of the tile.
	 */
	public JLabel labelF = new JLabel();

	/**
	 * Default constructor.
	 * 
	 * @param img
	 *            - tileset image
	 * @param x
	 *            - x coordinate on {@link TileMap}.
	 * @param y
	 *            - y coordinate on {@link TileMap}.
	 */
	public Tile(BufferedImage img, int x, int y) {
		super();
		this.img = img;
		this.x = x;
		this.y = y;
	}

	/**
	 * Paints itself on given {@link Graphics}.
	 * 
	 * @param g
	 *            - where to paint.
	 */
	public void paint(Graphics g) {
		int dx1 = x * WIDTH;
		int dx2 = dx1 + WIDTH;
		int dy1 = y * HEIGHT;
		int dy2 = dy1 + HEIGHT;
		int sx1 = 0;
		int sx2 = WIDTH;
		int sy1 = 0;

		// Flags decide which frame is used for displaying.
		if (start) {
			sy1 = HEIGHT * 2;
		} else if (goal) {
			sy1 = HEIGHT * 3;
		} else if (path) {
			sy1 = HEIGHT * 4;
		} else if (closed) {
			sy1 = HEIGHT * 5;
		} else if (next) {
			sy1 = HEIGHT * 6;
		} else if (blocking) {
			sy1 = HEIGHT;
		}
		int sy2 = sy1 + HEIGHT;

		g.drawImage(img, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, null);
	}

	/**
	 * @return the x
	 */
	public int getX() {
		return x;
	}

	/**
	 * @return the y
	 */
	public int getY() {
		return y;
	}

	/**
	 * @return the blocking
	 */
	public boolean isBlocking() {
		return blocking;
	}

	/**
	 * @param blocking
	 *            the blocking to set
	 */
	public void setBlocking(boolean blocking) {
		if (start || goal) {
			return;
		}
		this.blocking = blocking;
	}

	/**
	 * @return the start
	 */
	public boolean isStart() {
		return start;
	}

	/**
	 * @param start
	 *            the start to set
	 */
	public void setStart(boolean start) {
		this.start = start;
		blocking = false;
	}

	/**
	 * @return the goal
	 */
	public boolean isGoal() {
		return goal;
	}

	/**
	 * @param goal
	 *            the goal to set
	 */
	public void setGoal(boolean goal) {
		this.goal = goal;
		blocking = false;
	}

	/**
	 * Resets all flags to default values.
	 */
	public void resetFlags() {
		blocking = false;
		goal = false;
		start = false;
		resetPathFlags();
	}

	/**
	 * Reset flags related to pathfinding algorithm sets, including the score
	 * written in the label.
	 */
	public void resetPathFlags() {
		closed = false;
		path = false;
		next = false;
		labelF.setText("");
	}

	/**
	 * @param closed
	 *            the highlighted to set
	 */
	public void setClosed(boolean closed) {
		this.closed = closed;
	}

	/**
	 * @param path
	 *            the path to set
	 */
	public void setPath(boolean path) {
		this.path = path;
	}

	/**
	 * @param next
	 *            the next to set
	 */
	public void setNext(boolean next) {
		this.next = next;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (this.x == ((Tile) obj).x && this.y == ((Tile) obj).y)
			return true;
		return false;
	}

	/**
	 * Sets the text on label to a given value.
	 * 
	 * @param value
	 *            - given integer F score value.
	 */
	public void setLabelF(int value) {
		labelF.setText(Integer.toString(value));
	}
}